@@ -60,7 +60,7 @@ BEGIN {
 
    while (my ($k, $v) = each %ERR) {
       next if eval "Errno::$k ()";
-      warn "AnyEvent::Util: broken Errno module, adding Errno::$k.\n" if $AnyEvent::VERBOSE >= 8;
+      AE::log 8 => "AnyEvent::Util: broken Errno module, adding Errno::$k.";
 
       eval "sub Errno::$k () { $v }";
       push @Errno::EXPORT_OK, $k;
@@ -156,7 +156,7 @@ BEGIN {
       };
 
       *portable_socketpair = sub () {
-         socketpair my $fh1, my $fh2, Socket::AF_UNIX(), Socket::SOCK_STREAM(), Socket::PF_UNSPEC()
+         socketpair my $fh1, my $fh2, Socket::AF_UNIX(), Socket::SOCK_STREAM(), 0
             or return;
 
          ($fh1, $fh2)
@@ -223,8 +223,8 @@ my @fork_queue;
 
 sub _fork_schedule;
 sub _fork_schedule {
-   require Storable;
-   require POSIX;
+   require Storable unless $Storable::VERSION;
+   require POSIX    unless $Storable::VERSION;
 
    while ($forks < $MAX_FORKS) {
       my $job = shift @fork_queue
@@ -251,7 +251,9 @@ sub _fork_schedule {
          my $ww; $ww = AE::io $r, 0, sub {
             my $len = sysread $r, $buf, 65536, length $buf;
 
-            if ($len <= 0) {
+            return unless defined $len or $! != Errno::EINTR;
+
+            if (!$len) {
                undef $ww;
                close $r;
                --$forks;
@@ -293,7 +295,7 @@ sub _fork_schedule {
 
             $len = syswrite $w, $result, $len < 65536 ? $len : 65536, $ofs;
 
-            last if $len <= 0;
+            last unless $len || (!defined $len && $! == Errno::EINTR);
 
             $ofs += $len;
          }
@@ -365,8 +367,8 @@ BEGIN {
 
 =item $guard = guard { CODE }
 
-This function creates a special object that, when called, will execute the
-code block.
+This function creates a special object that, when called, will execute
+the code block.
 
 This is often handy in continuation-passing style code to clean up some
 resource regardless of where you break out of a process.
@@ -374,6 +376,11 @@ resource regardless of where you break out of a process.
 The L<Guard> module will be used to implement this function, if it is
 available. Otherwise a pure-perl implementation is used.
 
+While the code is allowed to throw exceptions in unusual conditions, it is
+not defined whether this exception will be reported (at the moment, the
+Guard module and AnyEvent's pure-perl implementation both try to report
+the error and continue).
+
 You can call one method on the returned object:
 
 =item $guard->cancel
@@ -383,30 +390,32 @@ guard.
 
 =cut
 
-if (!$ENV{PERL_ANYEVENT_AVOID_GUARD} && eval { require Guard; $Guard::VERSION >= 0.5 }) {
-   warn "AnyEvent::Util: using Guard module to implement guards.\n" if $AnyEvent::VERBOSE >= 8;
-   *guard = \&Guard::guard;
-} else {
-   warn "AnyEvent::Util: using pure-perl guard implementation.\n" if $AnyEvent::VERBOSE >= 8;
+BEGIN {
+   if (!$ENV{PERL_ANYEVENT_AVOID_GUARD} && eval { require Guard; $Guard::VERSION >= 0.5 }) {
+      *guard = \&Guard::guard;
+      AE::log 8 => "AnyEvent::Util: using Guard module to implement guards.";
+   } else {
+      *AnyEvent::Util::guard::DESTROY = sub {
+         local $@;
 
-   *AnyEvent::Util::guard::DESTROY = sub {
-      local $@;
+         eval {
+            local $SIG{__DIE__};
+            ${$_[0]}->();
+         };
 
-      eval {
-         local $SIG{__DIE__};
-         ${$_[0]}->();
+         AE::log 4 => "runtime error in AnyEvent::guard callback: $@" if $@;
       };
 
-      warn "runtime error in AnyEvent::guard callback: $@" if $@;
-   };
+      *AnyEvent::Util::guard::cancel = sub ($) {
+         ${$_[0]} = sub { };
+      };
 
-   *AnyEvent::Util::guard::cancel = sub ($) {
-      ${$_[0]} = sub { };
-   };
+      *guard = sub (&) {
+         bless \(my $cb = shift), "AnyEvent::Util::guard"
+      };
 
-   *guard = sub (&) {
-      bless \(my $cb = shift), "AnyEvent::Util::guard"
-   };
+      AE::log 8 => "AnyEvent::Util: using pure-perl guard implementation.";
+   }
 }
 
 =item AnyEvent::Util::close_all_fds_except @fds
@@ -429,7 +438,7 @@ Example: close all fds except 0, 1, 2.
 sub close_all_fds_except {
    my %except; @except{@_} = ();
 
-   require POSIX;
+   require POSIX unless $POSIX::VERSION;
 
    # some OSes have a usable /dev/fd, sadly, very few
    if ($^O =~ /(freebsd|cygwin|linux)/) {
@@ -574,7 +583,7 @@ in the callback and print key and certificate.
 sub run_cmd {
    my $cmd = shift;
 
-   require POSIX;
+   require POSIX unless $POSIX::VERSION;
 
    my $cv = AE::cv;
 
@@ -600,14 +609,16 @@ sub run_cmd {
             my $w; $w = AE::io $pr, 0,
                "SCALAR" eq ref $ob
                   ? sub {
-                       sysread $pr, $$ob, 16384, length $$ob
-                          and return;
+                       defined (sysread $pr, $$ob, 16384, length $$ob
+                                and return)
+                          or ($! == Errno::EINTR and return);
                        undef $w; $cv->end;
                     }
                   : sub {
                        my $buf;
-                       sysread $pr, $buf, 16384
-                          and return $ob->($buf);
+                       defined (sysread $pr, $buf, 16384
+                                and return $ob->($buf))
+                          or ($! == Errno::EINTR and return);
                        undef $w; $cv->end;
                        $ob->();
                     }
@@ -642,7 +653,9 @@ sub run_cmd {
             my $w; $w = AE::io $pw, 1, sub {
                my $len = syswrite $pw, $data;
 
-               if ($len <= 0) {
+               return unless defined $len or $! != Errno::EINTR;
+
+               if (!$len) {
                   undef $w; $cv->end;
                } else {
                   substr $data, 0, $len, "";
